"""
Global constants for MindsDB.
These values should not be user-configurable.
"""

# Default identifiers for single-tenant/local deployments
DEFAULT_COMPANY_ID = "00000000-0000-0000-0000-000000000000"
DEFAULT_USER_ID = "00000000-0000-0000-0000-000000000000"
